import 'util/HttpWrapper.dart';
import 'objects/Album.dart';
import 'objects/Gallery.dart';
import 'objects/Image.dart';

import 'dart:async';

/// Main class of Dart's imagur.
/// Allows to get album, galleries and allows to perform searching for gallery.
class Dartmagur {
  HttpWrapper _http;

  /// Creates new instance of [Dartmagur]. [clientSecret] parameters isn't required yet due library isn't using any `POST` endpoints.
  Dartmagur(String clientId, {String clientSecret: ""}) {
    _http = new HttpWrapper(clientId, clientSecret);
  }

  /// Returns album with given id.
  Future<Album> getAlbum(String id) async {
    var res = await _http.createRequest("GET", "/album/$id");

    return new Album(res.response['data'], _http);
  }

  /// Docs: https://apidocs.imgur.com/#311b0a17-29d2-48f5-b5cd-09dcc12863f5
  /// Returns List of galleries.
  Future<List<Gallery>> getGalleries({
    String section: "hot",
    String sort: "viral",
    int page: 0,
    String window: "day",
    bool showViral: true,
    bool mature: true,
  }) async {
    var res = await _http
        .createRequest("GET", "/gallery/$section/$sort/$window/$page", params: {
      "showViral": showViral.toString(),
      "mature": mature.toString(),
      "album_previews": "true"
    });

    var tmp = new List();
    res.response['data'].forEach((e) {
      tmp.add(new Gallery(e, _http));
    });

    return tmp;
  }

  /// Gets gallery based on given id
  Future<Gallery> getGallery(String id) async {
    var res = await _http.createRequest("GET", "/gallery/album/$id");

    return new Gallery(res.response['data'], _http);
  }

  /// Gets subreddit list of images
  Future<List<Image>> getSubreddit(String id,
      {String sort: "time", int page: 0, String window: "week"}) async {
    var res =
        await _http.createRequest("GET", "/gallery/r/$id/$sort/$window/$page");

    var tmp = new List<Image>();
    res.response['data'].forEach((o) {
      tmp.add(new Image(o, _http, null));
    });
    return tmp;
  }

  /// Search within Imagur for galley using [query]. All query objects are automatically ORed.
  Future<List<Gallery>> searchGallery(List<String> query,
      {String sort: "time", String window: "all", int page: 0}) async {
    String q = query.join(" OR ");
    var res = await _http.createRequest(
        "GET", "/gallery/search/$sort/$window/$page",
        params: {"q": q});

    var tmp = new List<Gallery>();
    res.response['data'].forEach((o) {
      tmp.add(new Gallery(o, _http));
    });

    return tmp;
  }
}
